package laboratoire4.pawns;

import laboratoire4.IPawn;

import java.awt.*;
import java.util.ArrayDeque;
import java.util.Queue;

public class PawnUtils {
    public static int distanceFromGoal(IPawn pawn) {
        return Math.abs(pawn.getPlayer().getGoal() - pawn.getRow());
    }

    public static int distanceFromHome(IPawn pawn) {
        return Math.abs(pawn.getRow() - pawn.getPlayer().getHome());
    }

    public static boolean areSamePlayers(IPawn a, IPawn b) {
        return a != null && b != null && a.getPlayer() == b.getPlayer();
    }

    public static boolean hasEasyWinPath(IPawn[][] board, IPawn initialPawn) {
        Queue<Point> positionsToVisit = new ArrayDeque<>();
        positionsToVisit.add(new Point(initialPawn.getRow(), initialPawn.getCol()));

        while (!positionsToVisit.isEmpty()) {
            Point pos = positionsToVisit.poll();

            for (PawnMovement movement : PawnMovement.values()) {
                if (!initialPawn.isMoveValid(board, movement, pos.x, pos.y)) {
                    continue;
                }

                int nextRow = pos.x + initialPawn.getDirection();
                int nextCol = pos.y + movement.getMove();

                IPawn pawn = board[nextRow][nextCol];
                if (pawn != null && pawn.getPlayer() != initialPawn.getPlayer() && pawn.isMoveValid(board, movement.getOpposite())) {
                    //  On peut se faire capturer en allant là !
                    continue;
                }

                if (nextRow == initialPawn.getPlayer().getGoal()) {
                    return true;
                }

                positionsToVisit.add(new Point(nextRow, nextCol));
            }
        }

        return false;
    }
}
