package laboratoire4.strategies;

import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnUtils;
import laboratoire4.pawns.SimulatedPawn;

import java.util.Collection;

public class AttackStrategy extends MiniMaxStrategy {
    private static final int ATTACK_DISTANCE_FROM_GOAL = 4;

    private long originalMinPawnCount;

    @Override
    public EvaluationResult getNextMove(Game game) {
        originalMinPawnCount = GameUtils.getMinPawnsAsStream(game.getBoard(), game.getPlayer()).count();

        return super.getNextMove(game);
    }

    @Override
    protected int evaluateSimulation() {
        for (SimulatedPawn pawn : getMaxPawns()) {
            if (pawn.getRow() == pawn.getPlayer().getGoal()) {
                return Integer.MAX_VALUE;
            }
        }

        int score = 0;

        int minPawnCount = getMinPawns().size();
        long capturedPawnCount = originalMinPawnCount - minPawnCount;
        score += Math.pow(capturedPawnCount, 2);

        return score;
    }

    @Override
    public int getWeight(Game game) {
        Collection<IPawn> maxPawns = GameUtils.getMaxPawns(game.getBoard(), game.getPlayer());

        int maxWeight = 0;
        for (IPawn pawn : maxPawns) {
            int weight = 0;

            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
            if (distanceFromGoal <= 1) {
                return WEIGHT_MAX;
            }

            if (distanceFromGoal == 2) {
                weight = PawnUtils.hasEasyWinPath(game.getBoard(), pawn) ? 8 : 5;
            }

            if (distanceFromGoal > 2 && distanceFromGoal <= ATTACK_DISTANCE_FROM_GOAL) {
                weight = 2;
            }

            if (weight > maxWeight) {
                maxWeight = weight;
            }
        }

        return maxWeight;
    }
}
