package laboratoire4.strategies;

import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnUtils;
import laboratoire4.pawns.SimulatedPawn;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class DefenseStrategy extends MiniMaxStrategy {
    private static final int DEFENSE_DISTANCE_FROM_HOME = 3;

    private final Map<IPawn, Integer> dangerousPawns = new HashMap<>();

    @Override
    public EvaluationResult getNextMove(Game game) {
        for (IPawn pawn : GameUtils.getMinPawns(game.getBoard(), game.getPlayer())) {
            int distance = PawnUtils.distanceFromGoal(pawn);

            if (distance <= DEFENSE_DISTANCE_FROM_HOME) {
                dangerousPawns.put(pawn, DEFENSE_DISTANCE_FROM_HOME - distance + 1);
            }
        }

        if (dangerousPawns.isEmpty()) {
            return null;
        }

        return super.getNextMove(game);
    }

    @Override
    public int getWeight(Game game) {
        Collection<IPawn> minPawns = GameUtils.getMinPawns(game.getBoard(), game.getPlayer());

        int maxWeight = 0;
        for (IPawn pawn : minPawns) {
            int weight = 0;

            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
            if (distanceFromGoal <= 1) {
                return WEIGHT_MAX;
            }

            if (distanceFromGoal == 2) {
                weight = PawnUtils.hasEasyWinPath(game.getBoard(), pawn) ? 8 : 5;
            }

            if (distanceFromGoal > 2 && distanceFromGoal <= DEFENSE_DISTANCE_FROM_HOME) {
                weight = 2;
            }

            if (weight > maxWeight) {
                maxWeight = weight;
            }
        }

        return maxWeight;
    }

    @Override
    protected int evaluateSimulation() {
        for (SimulatedPawn pawn : getMinPawns()) {
            if (pawn.getRow() == pawn.getPlayer().getGoal()) {
                return Integer.MIN_VALUE;
            }
        }


        int score = 0;

        for (IPawn dangerousPawn : dangerousPawns.keySet()) {
            Optional<SimulatedPawn> simulated = GameUtils.getMinPawnsAsStream(game.getBoard(), game.getPlayer())
                    .filter(p -> p.getOriginalRow() == dangerousPawn.getRow() && p.getOriginalCol() == dangerousPawn.getCol())
                    .findFirst();

            if (simulated.isEmpty()) {
                // Le pion a été capturé !
                score += Math.pow(dangerousPawns.get(dangerousPawn), 3);
            } else {
                // On est toujours en danger
                int distance = PawnUtils.distanceFromGoal(simulated.get());
                score -= Math.pow(DEFENSE_DISTANCE_FROM_HOME - distance + 1, 3);
            }
        }

        return score;
    }
}
