package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;

import java.util.Collection;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * Au début du jeu, il n'y a pas d'attaques, de défense ou de possibilités de gagner (pas besoin de minimax).
 * Alors, on avance quelques pions (pas tous) aléatoirement.
 */
public class StartingStrategy implements Strategy {
    private static final int PAWN_TO_MOVE = 3;
    private final Random random = new Random();

    @Override
    public EvaluationResult getNextMove(Game game) {
        Collection<IPawn> outsideHomePawns = GameUtils.getMaxPawnsAsStream(game.getBoard(), game.getPlayer())
                .filter(p -> p.getRow() != p.getPlayer().getHome() && p.getRow() != p.getPlayer().getHome() + p.getDirection())
                .collect(Collectors.toList());

        if (outsideHomePawns.size() > PAWN_TO_MOVE) {
            List<Action<IPawn>> validActions = Strategy.getValidActions(game.getBoard(), outsideHomePawns);
            if (!validActions.isEmpty()) {
                return getRandomMove(validActions);
            }
        }

        List<Action<IPawn>> validActions = Strategy.getValidActions(game);
        return getRandomMove(validActions);
    }

    @Override
    public int getWeight(Game game) {
        return 0;
    }

    private EvaluationResult getRandomMove(List<Action<IPawn>> actions) {
        int randomIndex = random.nextInt(actions.size());
        Action<IPawn> nextAction = actions.get(randomIndex);
        IPawn nextPawn = nextAction.getPawn();

        return new EvaluationResult(nextPawn.getRow(), nextPawn.getCol(), nextAction.getMovement());
    }
}
