package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.pawns.PawnMovement;

import java.util.*;
import java.util.stream.Collectors;

public interface Strategy {
    public static final int WEIGHT_MAX = 10;

    EvaluationResult getNextMove(Game game);
    int getWeight(Game game);

    static List<Action<IPawn>> getValidActions(Game game) {
        List<IPawn> maxPawns = Arrays.stream(game.getBoard())
                .flatMap(Arrays::stream)
                .filter(Objects::nonNull)
                .filter(p -> p.getPlayer() == game.getPlayer())
                .collect(Collectors.toList());

        return getValidActions(game.getBoard(), maxPawns);
    }

    static List<Action<IPawn>> getValidActions(IPawn[][] board, Collection<IPawn> pawns) {
        List<Action<IPawn>> validActions = new ArrayList<>();

        for (IPawn pawn : pawns) {
            for (PawnMovement movement : PawnMovement.values()) {
                if (pawn.isMoveValid(board, movement)) {
                    validActions.add(new Action<>(pawn, movement));
                }
            }
        }

        return validActions;
    }
}
