package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnMovement;

import java.util.Collection;
import java.util.stream.Collectors;

public class WinningStrategy implements Strategy {
    @Override
    public EvaluationResult getNextMove(Game game) {
        Action<IPawn> winningAction = getWinningAction(game);

        if (winningAction == null) {
            return null;
        }

        IPawn winningPawn = winningAction.getPawn();
        return new EvaluationResult(winningPawn.getRow(), winningPawn.getCol(), winningAction.getMovement());
    }

    @Override
    public int getWeight(Game game) {
        Action<IPawn> winningAction = getWinningAction(game);

        if (winningAction == null) {
            return 0;
        }

        return WEIGHT_MAX + 1;
    }

    private static Action<IPawn> getWinningAction(Game game) {
        Collection<IPawn> nearPawns = GameUtils.getMaxPawnsAsStream(game.getBoard(), game.getPlayer())
                .filter(p -> p.getRow() + p.getDirection() == p.getPlayer().getGoal())
                .collect(Collectors.toList());

        for (IPawn pawn : nearPawns) {
            for (PawnMovement movement : PawnMovement.values()) {
                if (pawn.isMoveValid(game.getBoard(), movement)) {
                    return new Action(pawn, movement);
                }
            }
        }

        return null;
    }
}
